/* C  function prototypes */

double box_mull( int *idum );
float  ranX( int *idum );
float  uniran( int *idum );
float  whran( int *idum );

/*
 *  ran3() and gasdev() are psuedo-random number generator functions
 *  Press, et.al.(1988). "Numerical Recipes in C." Cambridge University Press.
 */
double gasdev( int *idum );
float  ran3( int *idum );

/*
 * Brute Force (Trignometric) Box-Muller Transformation.
 */
double box_mull( idum )
int *idum; {

        static int iset=0;
        static double next;
        double rad,ang;

        if( iset == 0 ) {
            rad = sqrt( -2.0 * log( (double)ranX( idum ) ) );
            ang = 2.0 * 3.1415926536 * (double)ranX( idum );
            next = rad * cos( ang );
            iset = 1;
            return( rad * sin( ang ) );
            }
        else {
            iset=0;
            return next;
            }
        }

/*
 *      initseed()...initialize pseudo-random sequence using the Microsoft
 *                   long time() function to read system clock.
 */

int initseed() {
    time_t stime;
    int seed;

    time( &stime );
    seed = (int)stime ;
    if( seed > 0 )
        return( -seed );
    else if( seed == 0 )
        return( -13 );
    else return( seed );
    }

float ranX( idum )
int *idum; {
        extern rntype;
        if( rntype == 1 )
              return uniran( idum );
        else if( rntype == 2 )
              return whran( idum );
        else
              return ran3( idum );
        }

/*
 *  Random number generator based upon: Wichman and Hill(1982),
 *                                      Algorithm AS 183,
 *                                      Applied Statistics 31, 188-190.
 */

float whran( idum )
int *idum; {
        static int n1, n2, n3;
        static double nn1, nn2, nn3;
        static int ldum, ix=1, iy=1, iz=1; /* SEEDS */
        static int iff=0;
        double ixx, iyy, izz, rand;

        if( *idum < 0 || iff == 0 ) {
                ldum = (int)abs(*idum);
                *idum = 1;
                while( ldum > 30000 )
                    ldum = ldum%(30000);
                if( ldum < 1 )
                    ldum = 1;
                iff=1;
                nn1 = n1 = 30269;
                nn2 = n2 = 30307;
                nn3 = n3 = 30323;
                ix = iy = iz = ldum;
                }
        do {
            ix = 171 * (ix%177) -  2 * (ix/177);
            if( ix < 0 )
                ix += 30269;
            ixx = ix;

            iy = 172 * (iy%176) - 35 * (iy/176);
            if( iy < 0 )
                iy += 30307;
            iyy = iy;

            iz = 170 * (iz%178) - 63 * (iz/178);
            if( iz < 0 )
                iz += 30323;
            izz = iz;

            rand = fmod( ixx/nn1 + iyy/nn2 + izz/nn3, 1.0 );
            } while( rand==0.0 || rand==1.0 );
        return (float)rand;
        }

/*
 * Portable 32-bit random number generator proposed by Pierre
 * L'Ecuyer(1988), Communications of the ACM 31, 742-749 and 774.
 * Static integer seed values s1 and s2 must not only be >= 1
 * but also s1 <= 2,147,483,562 and s2 <= 2,147,483,398, respectively.
 */

float uniran( int *idum ) {

        static long s1, s2, ldum;
        static int iff=0;

        if( *idum < 0 || iff == 0 ) {
                ldum = (long)abs(*idum);
                if( ldum < 1L )
                    ldum = 1L;
                s1 = s2 = ldum;
                *idum = iff = 1;
                }

        ldum = s1 / 53668L;
        s1 =  40014L * ( s1 - ldum * 53668L ) - ldum * 12211L;
        if( s1 < 0 )
            s1 += 2147483563L;

        ldum = s2 / 52774L;
        s2 =  40692L * ( s2 - ldum * 52774L ) - ldum * 3791L;
        if( s2 < 0 )
            s2 += 2147483399L;

        ldum = s1 - s2;
        if( ldum < 1L )
            ldum += 2147483562L;

        return (float)( (double)ldum * 4.656613e-10 );
        }
